
// Helper stuff here for keyboard navigation


// this finds the next button to add hover state to, based on what arrow key was pressed 
function findNextButtonId(arrowKeyDirection, lastHoveredButtonID, lastHoveredButtonCoordinates) {
    // get new target coordinates
    var newTargetCoordinates = computeNewTargetCoordinates(arrowKeyDirection, lastHoveredButtonCoordinates);
    newX = newTargetCoordinates[0];
    newY = newTargetCoordinates[1];

    // loop through rows (Y)
    for (Y in gridRowsStringArray) {
        gridRowString = gridRowsStringArray[Y];
        var gridColumnStringArray = gridRowString.split(" ");
        // loop through columns (X)
        for (X in gridColumnStringArray) {
            // check if current coordinates in loop match our target coordinates
            if (X == newX && Y == newY) {
                // if coordinates match, check if we're still in the same button/box by comparing ids
                if (lastHoveredButtonID == gridColumnStringArray[X]) {
                    // if we're in the same box, repeat the arrow key direction command, by calling this function again but with the new target coordinates
                    return findNextButtonId(arrowKeyDirection, lastHoveredButtonID, [newX, newY]);
                }
                // if the coordinates match, but the ids don't, that means we're in a new button, so return the new buttons id and coordinates
                else {
                    return {
                        'lastHoveredButtonID': gridColumnStringArray[X],
                        'lastHoveredButtonCoordinates': [newX, newY]
                    };
                }
            }
        }
    }
    // after looping through coordinates/ids, if coordinates never match, it means we've gone beyond the grid and there is no new button
    // so return the same button id, and the original coordinates (lastHoveredButtonCoordinates)
    return {
        'lastHoveredButtonID': lastHoveredButtonID,
        'lastHoveredButtonCoordinates': lastHoveredButtonCoordinates
    };
}


// compute new target coordinates, depending on what direction is pressed
function computeNewTargetCoordinates(arrowKeyDirection, lastHoveredButtonCoordinates) {
    currentX = lastHoveredButtonCoordinates[0];
    currentY = lastHoveredButtonCoordinates[1];
    switch (arrowKeyDirection) {
        case 'up':
            newX = currentX;
            newY = currentY + 1;
            break;
        case 'down':
            newX = currentX;
            newY = currentY - 1;
            break;
        case 'left':
            newX = currentX - 1;
            newY = currentY;
            break;
        case 'right':
            newX = currentX + 1;
            newY = currentY;
            break;
        default:
            break;
    }
    return [newX, newY];
}


// setting some variables here
var oldButtonInfo = {
    'lastHoveredButtonID': 'box05',
    'lastHoveredButtonCoordinates': [4, 4]
};
var newButtonInfo = {
    'lastHoveredButtonID': 'box05',
    'lastHoveredButtonCoordinates': [4, 4]
};

// get the 'grid-template-areas' from CSS, to create grid for us to loop through
// use this grid to map box/button id's to grid coordinates. This is needed because there will be at least 4 coordinate pairs for every box/button
var gridRowsStringArray = [];
document.addEventListener('DOMContentLoaded', function () {
    var gridAreaWrapper = document.getElementsByClassName('wrapper')[0];
    var gridAreaString = window.getComputedStyle(gridAreaWrapper, null).getPropertyValue("grid-template-areas");
    var gridAreaStringArray = gridAreaString.split('" "');
    for (row in gridAreaStringArray) {
        gridRowString = gridAreaStringArray[row].replace('"', '');
        gridRowsStringArray.push(gridRowString);
    }
    gridRowsStringArray.reverse();
}, false);


// this is for use with the mouse only, it keeps the last hovered state in sync between the mouse and keyboard 
function getNewButtonCoordinates(newButtonId) {
    // loop through rows (Y)
    for (Y in gridRowsStringArray) {
        gridRowString = gridRowsStringArray[Y];
        var gridColumnStringArray = gridRowString.split(" ");
        // loop through columns (X)        (the column, gridColumnStringArray[X], is the button/box's id)
        for (X in gridColumnStringArray) {
            // check if newButtonId matches the currently looped over button id
            if (newButtonId == gridColumnStringArray[X]) {
                // if button id's match, return the coordinates of the new button 
                return {
                    'newButtonCoordinates': [parseInt(X), parseInt(Y)]
                };
            }
        }
    }
}

// Here we can do stuff when mouse is hovered over button
function mouseEnter(htmlElement) {
    // remove hover state/class from the old button
    document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
    // add hover state/class to new button 
    htmlElement.classList.add("mouseEnterHover");

    // save the ID of the newly hovered button in lastHoveredButtonID for future button hover changes
    oldButtonInfo.lastHoveredButtonID = htmlElement.id;
    // save coordinates of newly hovered button, so mouse and keyboard hover don't conflict with each other 
    var returnValue = getNewButtonCoordinates(htmlElement.id);
    oldButtonInfo.lastHoveredButtonCoordinates = returnValue.newButtonCoordinates;
}





function showLayout(layout) {
    for (let box in webview) {
        if (layout == "default") {
            element = document.getElementById(box);
            element.className = "box " + webview[box][layout].color;
            element.innerHTML = webview[box][layout].text;
        }
        else if (webview[box].hasOwnProperty(layout)) {
            element = document.getElementById(box);
            element.className = "box " + webview[box][layout].color;
            element.innerHTML = webview[box][layout].text;
        }
    }
}
document.addEventListener('DOMContentLoaded', function () {
    showLayout(layout);
});
function clickButton(htmlElement, layout) {
    //if layout is not default, check if button is configured for this alternate layout 
    if (layout != "default" && webview[htmlElement.id].hasOwnProperty(layout)) {
        // if so, call alternate layout function
        webview[htmlElement.id][layout].buttonAction();
    } else { // otherwise call defalut button/box function/method 
        webview[htmlElement.id]["default"].buttonAction();
    }
}


var layout = "default";
// Control Webview with Keyboard here
document.onkeydown = function (e) {
    switch (e.code) {
        case "ArrowUp":
            newButtonInfo = findNextButtonId('up', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;
        case "ArrowDown":
            newButtonInfo = findNextButtonId('down', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;
        case "ArrowLeft":
            newButtonInfo = findNextButtonId('left', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;
        case "ArrowRight":
            newButtonInfo = findNextButtonId('right', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;

        // using IJKL keys as arrow keys also
        case "KeyI": // up arrow
            newButtonInfo = findNextButtonId('up', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;
        case "KeyK": // down arrow
            newButtonInfo = findNextButtonId('down', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;
        case "KeyJ": // left arrow
            newButtonInfo = findNextButtonId('left', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;
        case "KeyL": // right arrow
            newButtonInfo = findNextButtonId('right', oldButtonInfo.lastHoveredButtonID, oldButtonInfo.lastHoveredButtonCoordinates);
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.remove("mouseEnterHover");
            document.getElementById(newButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            oldButtonInfo = newButtonInfo;
            break;
        case "MetaLeft":
        case "MetaRight":
        case "KeyS":
            // case "Space":
            if (layout == "default") {
                layout = "command";
                showLayout(layout);
            } else if (layout == "command") {
                layout = "default";
                showLayout(layout);
            } else {
                showLayout("default");
                layout = "command";
                showLayout(layout);
            }
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            break;
        case "AltLeft":
        case "AltRight":
        case "KeyA":
            if (layout == "default") {
                layout = "option";
                showLayout(layout);
            } else if (layout == "option") {
                layout = "default";
                showLayout(layout);
            } else {
                showLayout("default");
                layout = "option";
                showLayout(layout);
            }
            document.getElementById(oldButtonInfo.lastHoveredButtonID).classList.add("mouseEnterHover");
            break;
        case "F19": // using F19 as the default 'Enter' key to 'click' the box/button, this seems to help avoid accidental newlines. F19 is being used on Key Up inside BTT
            document.getElementsByClassName("mouseEnterHover")[0].click();
            break;
        case "Enter": // 'Enter' still works, but you have to press it manually, not on Key Up with BTT
            document.getElementsByClassName("mouseEnterHover")[0].click();
            break;
        case "Escape": // close webview without doing anything
            closeTheWebView();
            break;
        default:
            break; // exit this handler for other keys
    }
    e.preventDefault();
};